using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Diagnostics;
using WpfMsdn.MTPS;
using System.ServiceModel;
using System.Windows.Threading;
using System.Xml;
using System.Xml.XPath;
using System.Windows.Navigation;


namespace WpfMsdn
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>

    public partial class Window1 : Window
    {

        public Window1()
        {
            InitializeComponent();

            txtSearch.KeyDown += new KeyEventHandler(txtSearch_KeyDown);

            RequestNavigateEventHandler navigateHandler = OnNavigate;
            this.AddHandler(Hyperlink.RequestNavigateEvent, navigateHandler);
        }

        private void OnNavigate(object sender, RequestNavigateEventArgs e)
        {
            Debug.WriteLine(e.Uri);
            txtSearch.Text = e.Uri.ToString();
            DoSearch(e.Uri.ToString());
        }

        ContentServicePortType service;
        private void EnsureProxy()
        {
            lock (this)
            {
                if (service == null)
                {
                    ChannelFactory<ContentServicePortType> factory = new ChannelFactory<ContentServicePortType>("ContentService");
                    service = factory.CreateChannel();
                }
            }
        }

        void txtSearch_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Return)
            {
                DoSearch(txtSearch.Text);
            }
        }
        void DoSearch(string searchFor)
        {
            this.Cursor = Cursors.AppStarting;
            EventHandler dlg = delegate
            {
                EnsureProxy();
                getContentRequest request = new getContentRequest();
                request.contentIdentifier = searchFor;

                request.locale = System.Globalization.CultureInfo.CurrentUICulture.ToString();
                request.version = "VS.80";

                request.requestedDocuments = new requestedDocument[2];
                request.requestedDocuments[0] = new requestedDocument();
                request.requestedDocuments[0].type = documentTypes.primary;
                request.requestedDocuments[0].selector = "Mtps.Xhtml";
                request.requestedDocuments[1] = new requestedDocument();
                request.requestedDocuments[1].type = documentTypes.common;
                request.requestedDocuments[1].selector = "Mtps.Links";

                GetContentResponse response = null;
                try
                {
                    response = service.GetContent(new GetContentRequest(request));

                    Dispatcher.BeginInvoke(DispatcherPriority.Normal, (EventHandler)delegate
                    {
                        this.Cursor = null;

                        RenderResponse(response);
                    }, this, EventArgs.Empty);
                }
                catch (Exception ex)
                {
                    Dispatcher.BeginInvoke(DispatcherPriority.Normal, (EventHandler)delegate
                    {
                        this.Cursor = null;
                        SetErrorDoc(ex.Message);
                    }, this, EventArgs.Empty);
                }

            };

            dlg.BeginInvoke(this, EventArgs.Empty, delegate(IAsyncResult iar) { dlg.EndInvoke(iar); }, null);

        }

        private void SetErrorDoc(string message)
        {
            FlowDocument errorDoc = new FlowDocument();
            Paragraph errorParagraph = new Paragraph();
            errorParagraph.Inlines.Add(message);
            errorDoc.Blocks.Add(errorParagraph);
            docViewer.Document = errorDoc;
        }

        private void RenderResponse(GetContentResponse response)
        {
            XmlElement xhtml = GetXhtmlPrimaryDocument(response.getContentResponse);

            if (xhtml == null)
            {
                SetErrorDoc("Topic does not have any XHTML content.");
                return;
            }
            else
            {
                try
                {
                    // This should totally work, but doesn't. The reason appears to be that
                    // the element returned by the web service proxy has no owning document
                    // or has an owning document that's not quite right. As a result, there's 
                    // no match for the root, which means the transform fails. 
                    //XPathNavigator nav = xhtml.CreateNavigator();

                    // So instead I came up with this lame hack, which does
                    XmlDocument doc = new XmlDocument();
                    doc.PreserveWhitespace = true;
                    XmlNodeReader reader = new XmlNodeReader(xhtml);
                    doc.Load(reader);
                    XPathNavigator nav = doc.CreateNavigator();

                    TransformToFlowDocument tx = new TransformToFlowDocument(nav.NameTable);
                    tx.Execute(nav);
                    docViewer.Document = tx.GetDocument();
                    lblTitle.Content = tx.GetTitle();
                }
                catch (Exception ex)
                {
                    SetErrorDoc(ex.ToString());
                }
            }

        }
        private static XmlElement GetXhtmlPrimaryDocument(getContentResponse response)
        {
            XmlElement xhtml = null;
            foreach (primary primary in response.primaryDocuments)
            {
                if (primary.primaryFormat == "Mtps.Xhtml")
                {
                    xhtml = primary.Any;
                }
            }

            return xhtml;
        }
    }
}