using System;
using System.Reflection;
using System.Xml;
using System.Xml.XPath;


namespace MsdnMan.XsltObjects
{
    public class Transform
    {
        private XPathNavigator _context;
        private IXmlNamespaceResolver _namespaceResolver; 
        private readonly XPathMethodInfoPairCollection _xpathMethodInfos = new XPathMethodInfoPairCollection();

        public Transform()
        {
        }

        protected IXmlNamespaceResolver NamespaceResolver
        {
            get { return _namespaceResolver; }
            set { _namespaceResolver = value; }
        }

        public void Execute(XPathNavigator context)
        {
            InitializeMethodMap(context);

            _context = context;

            ApplyTemplates("/");

        }

        protected void ApplyTemplates()
        {
            ApplyTemplates("*"); 
        }

        protected void ApplyTemplates(string xpath)
        {
            XPathNodeIterator nodes = _context.Select(Compile(_context, xpath));

            while (nodes.MoveNext())
            {
                XPathNavigator oldContext = _context; 
                _context = nodes.Current; 
                foreach (XPathMethodInfoPair pair in _xpathMethodInfos)
                {
                    if (_context.Matches(pair.Expression))
                    {
                        pair.Method.Invoke(this, new object[] { _context });
                        break;
                    }
                }
                _context = oldContext; 
            }
        }

        [Match("*|/")]
        protected virtual void DefaultRule(XPathNavigator context)
        {
            ApplyTemplates("*");
        }

        private void InitializeMethodMap(XPathNavigator context)
        {
            foreach (MethodInfo method in GetType().GetMethods(BindingFlags.Public | BindingFlags.Instance | 
                BindingFlags.FlattenHierarchy | BindingFlags.NonPublic))
            {
                if (method.IsDefined(typeof(MatchAttribute), true))
                {
                    foreach (MatchAttribute match in method.GetCustomAttributes(typeof(MatchAttribute), false))
                    {
                        XPathExpression expression = Compile(context, match.XPath);
                        _xpathMethodInfos.Add(new XPathMethodInfoPair(expression, method)); 
                    }
                }
            }
        }

        private XPathExpression Compile(XPathNavigator context, string xpath)
        {
            XPathExpression expression = context.Compile(xpath);
            if (NamespaceResolver != null)
            {
                expression.SetContext(NamespaceResolver);
            }
            return expression;
        }
    }
}
